
        list    p=16C84
        constant RESETVECT = 0x0000
	constant PROGMEM = 0x0005

;*******************************************************************
;                             ESTOP.ASM
;              EMERGENCY STOP PROGRAM FOR MOBILE ROBOT
;*******************************************************************
;
;	INPUTS AND OUTPUTS
;	DIRECTION	PORT	PIN	DESIGNATOR
;	---------------------------------------------------
;	inputs		portb	 0 	INPULSE (pulse input)
;			portb	 1	LESTOP (local EStop button)
;			portb	 2	LGO (local GO button)	
;
;	outputs		portb	 3	ESTOP (emergency stop)
;			portb	 4	GO (procede)
;			portb	 5	LC (lost carrier)
;
;*******************************************************************

INDIRECT equ    0x00
RTCC    equ     0x01
PC      equ     0x02
STATUS  equ     0x03
FSR     equ     0x04
PORTA   equ     0x05
PORTB   equ     0x06
PORTC   equ     0x07
flags   equ     0x08            ; Misc flags
GoCount equ     0x09            ; General purpose temporary register
EstpCnt equ     0x0A
delay1  equ     0x0B           	; Delay between tests of input
delay2  equ     0x0C            ; Delay between frames
msCount equ     0x0D


CARRY   equ     0               ; Status Register bits
DCARRY  equ     1
ZFLAG   equ     2
PDFLAG  equ     3
TOFLAG  equ     4
PA0     equ     5
PA1     equ     6
PA2     equ     7

W       equ     0               ; W register as destination
SAME	equ	1
LSB     equ     0


			; flag register bits
BPSF	equ	0		; bad pulse short flag - to short
GOF	equ	1		; right button flag - short pulse = go
NORMF	equ	2		; normal pulse flag - no butons pressed = do nothing
ESTOPF	equ	3		; left button flag - long pulse = emergency stop
BPLF	equ	4		; bad pulse long flag - to long
BFF	equ	5		; bad frame flag - not recieving pulse start
LESTOPF	equ	6		; local ESTOP flag - local ESTOP button press
LGOF	equ	7		; local GO flag - local GO button press


			; portb pin assignments
INPULSE	equ	0		; pulse input
LESTOP	equ	1		; local ESTOP button input
LGO	equ	2		; local GO button input
ESTOP	equ	3		; emergency stop output
GO	equ	4		; procede output
LC	equ	5		; lost carrier output

PORTBIO	equ	0xC7		;3 inputs=tris; 

DefaultDelay1	equ	d'3'	; delay about 0.01 mS
DefaultDelay2	equ	d'13'	; dealy about 0.08 mS

	;signal region widths: thresholds*0.01 mS
BPSTHD	equ	d'53'
GOTHD	equ	d'80'
NORMTHD	equ	d'113'
ESTPTHD	equ	d'140'
   ; any pulse longer than ESTPTHD is in BPL region

NumHits	equ	d'10'		; number of hits in each region before action

	; signal frame thresholds*0.08mS
SFRAME	equ	d'90'		; short frame limit = 12 ms
LFRAME	equ	d'110'		; long frame limit = 14 ms






	org     RESETVECT
        goto    start

        org     PROGMEM


start   clrf    FSR
	clrf	flags
	clrf	msCount

        clrf    PORTB		; setup IO
        movlw   PORTBIO		
        tris    PORTB           

 	bsf	flags,ESTOPF	; wake up with ESTOP on
	bsf	PORTB,ESTOP
	

	;not needed?
	;bsf	flags,LCF	; wake up with lost carrier
	bsf	PORTB,LC



fstlow	btfsc	PORTB,INPULSE	; wait for rising edge on input
	goto	fstlow
fstrise	btfss	PORTB,INPULSE
	goto	fstrise

ResetWaitFall
	call	ResetHits
waitFall	
	clrf	msCount		; reset the counter	
;wfLoop	movlw   DefaultDelay1   ; Load time unit = 0.01 mS
;	call	DelaySub	; delay - count - check rise
wfLoop	incfsz	msCount,SAME
	goto 	wfSkip

	bsf	PORTB,LC	; Timeout on msCount
	goto	fstlow

wfSkip	btfsc	PORTB,INPULSE	; wait for falling edge on input
	goto 	wfLoop


classify
BPStest
	movlw	BPSTHD		; get the threshold
	subwf	msCount,W
	btfsc	STATUS,CARRY	; measure for BPS region
	goto	gotest		; next measurment
			; clear / set outputs
	bsf	PORTB,LC	; set lost carrier
	goto	fstrise

gotest
	movlw	GOTHD		; get the threshold
	subwf	msCount,W
	btfsc	STATUS,CARRY	; measure for GO region
	goto	normtest	; next measurment
			; clear / set flags   
	bsf	flags,GOF	; report a go	
	bcf	flags,NORMF	
	bcf	flags,ESTOPF	
	goto	waitrise

normtest
	movlw	NORMTHD		; get the threshold
	subwf	msCount,W
	btfsc	STATUS,CARRY	; measure for NORM region
	goto	estptest	; next measurment
			; clear / set flags   
	bcf	flags,GOF		
	bsf	flags,NORMF	; report a norm
	bcf	flags,ESTOPF
	goto	waitrise


	
estptest
	movlw	ESTPTHD		; get the threshold
	subwf	msCount,W
	btfsc	STATUS,CARRY	; measure for ESTOP region
	goto	bpltest		; next measurment

	bcf	flags,GOF		
	bcf	flags,NORMF
	bsf	flags,ESTOPF	; report an ESTOP
	goto	waitrise


bpltest		; if here than pulse=too long
	bsf	PORTB,LC	; set lost carrier
	goto	fstrise
			; wait for pulse to fall again
waitrise
	rrf	msCount		; Divide msCount by 8
	rrf	msCount	
	rrf	msCount	
	movlw	0x1F
	andwf	msCount
	btfsc	STATUS,CARRY	; Round
	incf	msCount

wrLoop	movlw   DefaultDelay2   ; Load time unit = 0.01 mS
	call	DelaySub	; delay - count - check rise
	incfsz	msCount,SAME
	goto 	wrSkip

;	decf	msCount,SAME
	bsf	PORTB,LC
	goto	fstrise

wrSkip	btfss	PORTB,INPULSE	; wait for rising edge on input
	goto 	wrLoop


sfTest	movlw	SFRAME		; get the threshold
	subwf	msCount,W
	btfsc	STATUS,CARRY	; measure for short frame
	goto	lfTest		; next measurment

	bsf	PORTB,LC	; set lost carrier
	goto	ResetWaitFall


lfTest	movlw	LFRAME		; get the threshold
	subwf	msCount,W
	btfss	STATUS,CARRY	; measure for long frame
	goto	okFrame		; next measurment

	bsf	PORTB,LC	; set lost carrier
	goto	ResetWaitFall

okFrame	bcf	PORTB,LC	; clear lost carrier 
	
goFtest
	btfss	flags,GOF
	goto	normFtest
	movlw	NumHits
	movwf	EstpCnt

	decfsz	GoCount
	goto	waitFall	

	bcf	PORTB,ESTOP
	bsf	PORTB,GO
	goto	ResetWaitFall

normFtest
	btfss	flags,NORMF
	goto	estopFtest

	bcf	PORTB,GO
	goto	ResetWaitFall

estopFtest
	movlw	NumHits
	movwf	GoCount
	decfsz	EstpCnt
	goto	waitFall

	bcf	PORTB,GO
	bsf	PORTB,ESTOP
	goto	ResetWaitFall


ResetHits
	movlw	NumHits
	movwf	GoCount
	movwf	EstpCnt
	retlw	0

	
	
DelaySub			; delay for amount indicated in W
	movwf   delay1		
wait	decfsz	delay1,1	; count down
	goto	wait
	retlw	0		; done delay

	

	END
